import java.io.*;
import java.net.*;

public class DSFClient extends Thread
{ final String LOCAL_HOST = "localhost";
  final int    LOCAL_PORT = 9548;
  
  Socket           clientSocket;
  DataOutputStream outToServer;
  boolean          isActive = true;
    
  public DSFClient()
  { super();
    connectToServer(LOCAL_HOST, LOCAL_PORT);
  }

  public DSFClient(String host, int port)
  { super();
    connectToServer(host, port);
  }

  void connectToServer(String host, int port)
  { try
    { System.out.print("Connecting to " + host + ":" + port + " . . . ");
      clientSocket = new Socket(host, port);      
    }
    catch (UnknownHostException uhe)
    { System.out.println("Unknown host " + host + ":" + port);
    }
    catch(IOException ioe)
    { System.out.println("failed");
      System.exit(0);
    }
    System.out.println("success");
  }

  public void sendMsg(String msg)
  { try
    { outToServer = new DataOutputStream(clientSocket.getOutputStream());
      outToServer.writeBytes(msg);
      outToServer.flush();
    }
    catch (IOException ioe)
    { System.out.println("Problems sending to server");
      try
      { clientSocket.shutdownInput();          	
      }
      catch (IOException ioe2)
      { ioe2.printStackTrace();
        System.exit(-1);
      }
    }
  }

  public void run()
  { BufferedReader inFromServer;
    InputStream    is;
    String         msg;

    while(!clientSocket.isInputShutdown())
    { try
      { if((is = clientSocket.getInputStream()) != null)
        { inFromServer = new BufferedReader(new InputStreamReader(is));
          while ((msg = inFromServer.readLine())!=null)
          { System.out.println(msg);
          }
        } 
      }
      catch(IOException ioe)
      { if (clientSocket.isConnected())
        { System.out.println("Problems receiving message from server");
          try
          { clientSocket.shutdownInput();          	
          }
          catch (IOException ioe2)
          { ioe2.printStackTrace();
            System.exit(-1);
          }
        }
      }
      if (clientSocket.isOutputShutdown())
      { System.out.println("Out closed");
        try
        { clientSocket.shutdownInput();          	
        }
        catch (IOException ioe2)
        { ioe2.printStackTrace();
          System.exit(-1);
        }
      }
    }
    System.out.println("Connection closed");
  }

  public void closeSocket()
  { yield();
    if (!clientSocket.isInputShutdown())
    { try
      { clientSocket.close();
      }
      catch(IOException ioe)
      { System.out.println("Error closing socket");
      }
    }
  }
   

  public static void main(String argv[])
  { DSFClient rc;

    if (argv.length < 2)
    { rc = new DSFClient();
    }
    else
    { rc = new DSFClient(argv[0], Integer.parseInt(argv[1]));
    }

    rc.start();
    String         sentence;
    BufferedReader inFromUser = 
      new BufferedReader(new InputStreamReader(System.in));
    
    while(true)
    { try
      { sentence = inFromUser.readLine();
        if(sentence==null) break;
        sentence += "\n";
        System.out.print("SENDING: " + sentence);
        rc.sendMsg(sentence);
      }
      catch(IOException ioe)
      { System.out.println("User not providing input to server");
      }
    }
    System.out.println("Exiting . . .");

	  rc.closeSocket();
  }
}
